@@ -7,24 +7,27 @@ from os import makedirs
 from os.path import basename, join, normpath
 from pathlib import Path
 from shutil import rmtree
-from typing import Any, Dict, Optional, Tuple, Union, cast
+from typing import Any, Dict, List, Optional, Tuple, Union, cast
 
 import attr
 import numpy as np
 import wkw
 
+from webknossos.dataset._utils.infer_bounding_box_existing_files import (
+    infer_bounding_box_existing_files,
+)
 from webknossos.geometry import BoundingBox, Vec3Int
-from webknossos.utils import get_executor_for_args
+from webknossos.utils import copy_directory_with_symlinks, get_executor_for_args
 
 from .layer import (
     Layer,
-    LayerCategories,
     SegmentationLayer,
     _dtype_per_channel_to_element_class,
     _dtype_per_layer_to_dtype_per_channel,
     _normalize_dtype_per_channel,
     _normalize_dtype_per_layer,
 )
+from .layer_categories import COLOR_CATEGORY, SEGMENTATION_CATEGORY, LayerCategoryType
 from .properties import (
     DatasetProperties,
     DatasetViewConfiguration,
@@ -49,52 +52,15 @@ def _copy_job(args: Tuple[View, View, int]) -> None:
 
 class Dataset:
     """
-    A dataset is the entry point of the Dataset API. An existing dataset on disk can be opened
+    A dataset is the entry point of the Dataset API.
+    An existing dataset on disk can be opened
     or new datasets can be created.
 
-    A `Dataset` stores the data in `.wkw` files on disk.
+    A dataset stores the data in `.wkw` files on disk with metadata in `datasource-properties.json`.
+    The information in those files are kept in sync with the object.
 
-    ## Examples
-
-    ### Creating Datasets
-    ```python
-    from webknossos.dataset.dataset import Dataset
-
-    dataset = Dataset.create(<path_to_new_dataset>, scale=(1, 1, 1))
-    # Adds a new layer
-    layer = dataset.add_layer(
-        layer_name="color",
-        category=LayerCategories.COLOR_TYPE,
-        dtype_per_channel="uint8",
-        num_channels=3
-    )
-    # Adds an existing layer from a different dataset
-    sym_layer = dataset.add_symlink_layer(<foreign_layer_path>)
-    ```
-
-    ### Opening Datasets
-    ```python
-    from webknossos.dataset.dataset import Dataset
-
-    dataset = Dataset(<path_to_dataset>)
-    # Assuming that the dataset has a layer called 'color'
-    layer = dataset.get_layer("color")
-    ```
-
-    ### Copying Datasets
-    ```python
-    from webknossos.dataset.dataset import Dataset
-
-    dataset = Dataset(<path_to_dataset>)
-    # Copying the dataset with different block_len and file_len
-    copy_of_dataset = dataset.copy_dataset(
-        <path_to_new_dataset>,
-        block_len=8,
-        file_len=8
-    )
-    ```
-
-    ## Functions
+    Each dataset consists of one or more layers (webknossos.dataset.layer.Layer),
+    which themselves can comprise multiple magnifications (webknossos.dataset.mag_view.MagView).
     """
 
     def __init__(self, dataset_path: Union[str, Path]) -> None:
@@ -147,6 +113,11 @@ class Dataset:
         """
         return self._layers
 
+    def upload(self) -> str:
+        from webknossos.client._upload_dataset import upload_dataset
+
+        return upload_dataset(self)
+
     def get_layer(self, layer_name: str) -> Layer:
         """
         Returns the layer called `layer_name` of this dataset. The return type is `webknossos.dataset.layer.Layer`.
@@ -162,10 +133,10 @@ class Dataset:
     def add_layer(
         self,
         layer_name: str,
-        category: str,
-        dtype_per_layer: Union[str, np.dtype, type] = None,
-        dtype_per_channel: Union[str, np.dtype, type] = None,
-        num_channels: int = None,
+        category: LayerCategoryType,
+        dtype_per_layer: Optional[Union[str, np.dtype, type]] = None,
+        dtype_per_channel: Optional[Union[str, np.dtype, type]] = None,
+        num_channels: Optional[int] = None,
         **kwargs: Any,
     ) -> Layer:
         """
@@ -228,10 +199,10 @@ class Dataset:
             data_format="wkw",
         )
 
-        if category == LayerCategories.COLOR_TYPE:
+        if category == COLOR_CATEGORY:
             self._properties.data_layers += [layer_properties]
             self._layers[layer_name] = Layer(self, layer_properties)
-        elif category == LayerCategories.SEGMENTATION_TYPE:
+        elif category == SEGMENTATION_CATEGORY:
             assert (
                 "largest_segment_id" in kwargs
             ), f"Failed to create segmentation layer {layer_name}: the parameter 'largest_segment_id' was not specified, which is necessary for segmentation layers."
@@ -252,7 +223,7 @@ class Dataset:
             )
         else:
             raise RuntimeError(
-                f"Failed to add layer ({layer_name}) because of invalid category ({category}). The supported categories are '{LayerCategories.COLOR_TYPE}' and '{LayerCategories.SEGMENTATION_TYPE}'"
+                f"Failed to add layer ({layer_name}) because of invalid category ({category}). The supported categories are '{COLOR_CATEGORY}' and '{SEGMENTATION_CATEGORY}'"
             )
 
         self._export_as_json()
@@ -261,10 +232,10 @@ class Dataset:
     def get_or_add_layer(
         self,
         layer_name: str,
-        category: str,
-        dtype_per_layer: Union[str, np.dtype, type] = None,
-        dtype_per_channel: Union[str, np.dtype, type] = None,
-        num_channels: int = None,
+        category: LayerCategoryType,
+        dtype_per_layer: Optional[Union[str, np.dtype, type]] = None,
+        dtype_per_channel: Optional[Union[str, np.dtype, type]] = None,
+        num_channels: Optional[int] = None,
         **kwargs: Any,
     ) -> Layer:
         """
@@ -337,17 +308,41 @@ class Dataset:
         layer_properties.name = layer_name
 
         self._properties.data_layers += [layer_properties]
-        if layer_properties.category == LayerCategories.COLOR_TYPE:
+        if layer_properties.category == COLOR_CATEGORY:
             self._layers[layer_name] = Layer(self, layer_properties)
-        elif layer_properties.category == LayerCategories.SEGMENTATION_TYPE:
+        elif layer_properties.category == SEGMENTATION_CATEGORY:
             self._layers[layer_name] = SegmentationLayer(self, layer_properties)
         else:
             raise RuntimeError(
-                f"Failed to add layer ({layer_name}) because of invalid category ({layer_properties.category}). The supported categories are '{LayerCategories.COLOR_TYPE}' and '{LayerCategories.SEGMENTATION_TYPE}'"
+                f"Failed to add layer ({layer_name}) because of invalid category ({layer_properties.category}). The supported categories are '{COLOR_CATEGORY}' and '{SEGMENTATION_CATEGORY}'"
             )
         self._export_as_json()
         return self._layers[layer_name]
 
+    def add_layer_for_existing_files(
+        self, layer_name: str, category: LayerCategoryType, **kwargs: Any
+    ) -> Layer:
+        assert layer_name not in self.layers, f"Layer {layer_name} already exists!"
+        mag_headers = list((self.path / layer_name).glob("*/header.wkw"))
+
+        assert (
+            len(mag_headers) != 0
+        ), f"Could not find any header.wkw files in {self.path / layer_name}, cannot add layer."
+        with wkw.Dataset.open(str(mag_headers[0].parent)) as wkw_dataset:
+            header = wkw_dataset.header
+        layer = self.add_layer(
+            layer_name,
+            category=category,
+            num_channels=header.num_channels,
+            dtype_per_channel=header.voxel_type,
+            **kwargs,
+        )
+        for mag_dir in layer.path.iterdir():
+            layer.add_mag_for_existing_files(mag_dir.name)
+        min_mag_view = layer.mags[min(layer.mags)]
+        layer.bounding_box = infer_bounding_box_existing_files(min_mag_view)
+        return layer
+
     def get_segmentation_layer(self) -> SegmentationLayer:
         """
         Returns the only segmentation layer.
@@ -356,7 +351,7 @@ class Dataset:
         """
         return cast(
             SegmentationLayer,
-            self._get_layer_by_category(LayerCategories.SEGMENTATION_TYPE),
+            self._get_layer_by_category(SEGMENTATION_CATEGORY),
         )
 
     def get_color_layer(self) -> Layer:
@@ -365,7 +360,7 @@ class Dataset:
 
         Fails with a RuntimeError if there are multiple color layers or none.
         """
-        return self._get_layer_by_category(LayerCategories.COLOR_TYPE)
+        return self._get_layer_by_category(COLOR_CATEGORY)
 
     def delete_layer(self, layer_name: str) -> None:
         """
@@ -388,7 +383,7 @@ class Dataset:
         self,
         foreign_layer: Union[str, Path, Layer],
         make_relative: bool = False,
-        new_layer_name: str = None,
+        new_layer_name: Optional[str] = None,
     ) -> Layer:
         """
         Creates a symlink to the data at `foreign_layer` which belongs to another dataset.
@@ -416,7 +411,7 @@ class Dataset:
         foreign_layer_symlink_path = (
             Path(os.path.relpath(foreign_layer_path, self.path))
             if make_relative
-            else Path(os.path.abspath(foreign_layer_path))
+            else foreign_layer_path.resolve()
         )
         os.symlink(foreign_layer_symlink_path, join(self.path, layer_name))
         original_layer = Dataset(foreign_layer_path.parent).get_layer(
@@ -433,7 +428,9 @@ class Dataset:
         return self.layers[layer_name]
 
     def add_copy_layer(
-        self, foreign_layer: Union[str, Path, Layer], new_layer_name: str = None
+        self,
+        foreign_layer: Union[str, Path, Layer],
+        new_layer_name: Optional[str] = None,
     ) -> Layer:
         """
         Copies the data at `foreign_layer` which belongs to another dataset to the current dataset.
@@ -446,7 +443,6 @@ class Dataset:
         else:
             foreign_layer_path = Path(foreign_layer)
 
-        foreign_layer_path = Path(os.path.abspath(foreign_layer_path))
         foreign_layer_name = foreign_layer_path.name
         layer_name = (
             new_layer_name if new_layer_name is not None else foreign_layer_name
@@ -474,8 +470,8 @@ class Dataset:
         self,
         new_dataset_path: Union[str, Path],
         scale: Optional[Tuple[float, float, float]] = None,
-        block_len: int = None,
-        file_len: int = None,
+        block_len: Optional[int] = None,
+        file_len: Optional[int] = None,
         compress: Optional[bool] = None,
         args: Optional[Namespace] = None,
     ) -> "Dataset":
@@ -535,11 +531,41 @@ class Dataset:
         new_ds._export_as_json()
         return new_ds
 
-    def _get_layer_by_category(self, category: str) -> Layer:
-        assert (
-            category == LayerCategories.COLOR_TYPE
-            or category == LayerCategories.SEGMENTATION_TYPE
+    def shallow_copy_dataset(
+        self,
+        new_dataset_path: Path,
+        name: Optional[str] = None,
+        make_relative: bool = False,
+        layers_to_ignore: Optional[List[str]] = None,
+    ) -> "Dataset":
+        """
+        Create a new dataset at the given path. Link all mags of all existing layers.
+        In addition, link all other directories in all layer directories
+        to make this method robust against additional files e.g. layer/mappings/agglomerate_view.hdf5.
+        This method becomes useful when exposing a dataset to webknossos.
+        """
+        new_dataset = Dataset.create(
+            new_dataset_path, scale=self.scale, name=name or self.name
         )
+        for layer_name, layer in self.layers.items():
+            if layers_to_ignore is not None and layer_name in layers_to_ignore:
+                continue
+            new_layer = new_dataset.add_layer_like(layer, layer_name)
+            for mag_view in layer.mags.values():
+                new_layer.add_symlink_mag(mag_view, make_relative)
+
+            # copy all other directories with a dir scan
+            copy_directory_with_symlinks(
+                layer.path,
+                new_layer.path,
+                ignore=[str(mag) for mag in layer.mags] + [PROPERTIES_FILE_NAME],
+                make_relative=make_relative,
+            )
+
+        return new_dataset
+
+    def _get_layer_by_category(self, category: LayerCategoryType) -> Layer:
+        assert category == COLOR_CATEGORY or category == SEGMENTATION_CATEGORY
 
         layers = [layer for layer in self.layers.values() if category == layer.category]
 
@@ -664,9 +690,9 @@ class Dataset:
             )
 
     def _initialize_layer_from_properties(self, properties: LayerProperties) -> Layer:
-        if properties.category == LayerCategories.COLOR_TYPE:
+        if properties.category == COLOR_CATEGORY:
             return Layer(self, properties)
-        elif properties.category == LayerCategories.SEGMENTATION_TYPE:
+        elif properties.category == SEGMENTATION_CATEGORY:
             return SegmentationLayer(self, properties)
         else:
             raise RuntimeError(
