@@ -2,7 +2,7 @@ import math
 import warnings
 from pathlib import Path
 from types import TracebackType
-from typing import Callable, Optional, Tuple, Type, Union
+from typing import TYPE_CHECKING, Any, Callable, Dict, Optional, Tuple, Type, Union
 
 import cluster_tools
 import numpy as np
@@ -12,6 +12,10 @@ from wkw import Dataset, wkw
 from webknossos.geometry import BoundingBox, Vec3Int, Vec3IntLike
 from webknossos.utils import wait_and_ensure_success
 
+if TYPE_CHECKING:
+    from webknossos.dataset._utils.buffered_slice_reader import BufferedSliceReader
+    from webknossos.dataset._utils.buffered_slice_writer import BufferedSliceWriter
+
 
 class View:
     """
@@ -34,14 +38,13 @@ class View:
         """
         Do not use this constructor manually. Instead use `webknossos.dataset.mag_view.MagView.get_view()` to get a `View`.
         """
-        self._dataset: Optional[Dataset] = None
         self._path = path_to_mag_view
         self._header: wkw.Header = header
         self._size: Vec3Int = Vec3Int(size)
         self._global_offset: Vec3Int = Vec3Int(global_offset)
         self._is_bounded = is_bounded
         self._read_only = read_only
-        self._is_opened = False
+        self._cached_wkw_dataset = None
         # The bounding box of the view is used to prevent warnings when writing compressed but unaligned data
         # directly at the borders of the bounding box.
         # A View is unable to get this information from the Dataset because it is detached from it.
@@ -69,37 +72,6 @@ class View:
     def read_only(self) -> bool:
         return self._read_only
 
-    def open(self) -> "View":
-        """
-        Opens the actual handles to the data on disk.
-        A `MagDataset` has to be opened before it can be read or written to. However, the user does not
-        have to open it explicitly because the API automatically opens it when it is needed.
-        The user can choose to open it explicitly to avoid that handles are opened and closed automatically
-        each time data is read or written.
-        """
-        if self._is_opened:
-            raise Exception("Cannot open view: the view is already opened")
-        else:
-            self._dataset = Dataset.open(
-                str(self._path)
-            )  # No need to pass the header to the wkw.Dataset
-            self._is_opened = True
-        return self
-
-    def close(self) -> None:
-        """
-        Complementary to `open`, this closes the handles to the data.
-
-        See `open` for more information.
-        """
-        if not self._is_opened:
-            raise Exception("Cannot close View: the view is not opened")
-        else:
-            assert self._dataset is not None  # because the View was opened
-            self._dataset.close()
-            self._dataset = None
-            self._is_opened = False
-
     def write(
         self,
         data: np.ndarray,
@@ -116,7 +88,6 @@ class View:
 
         offset = Vec3Int(offset)
 
-        was_opened = self._is_opened
         # assert the size of the parameter data is not in conflict with the attribute self.size
         data_dims = Vec3Int(data.shape[-3:])
         _assert_positive_dimensions(offset, data_dims)
@@ -131,14 +102,7 @@ class View:
         if self._is_compressed():
             absolute_offset, data = self._handle_compressed_write(absolute_offset, data)
 
-        if not was_opened:
-            self.open()
-        assert self._dataset is not None  # because the View was opened
-
-        self._dataset.write(absolute_offset.to_np(), data)
-
-        if not was_opened:
-            self.close()
+        self._wkw_dataset.write(absolute_offset.to_np(), data)
 
     def read(
         self,
@@ -202,23 +166,14 @@ class View:
         absolute_offset: Vec3Int,
         size: Vec3Int,
     ) -> np.ndarray:
-        was_opened = self._is_opened
-        if not was_opened:
-            self.open()
-        assert self._dataset is not None  # because the View was opened
-
-        data = self._dataset.read(absolute_offset.to_np(), size.to_np())
-
-        if not was_opened:
-            self.close()
-
+        data = self._wkw_dataset.read(absolute_offset.to_np(), size.to_np())
         return data
 
     def get_view(
         self,
         offset: Vec3IntLike = Vec3Int(0, 0, 0),
         size: Optional[Vec3IntLike] = None,
-        read_only: bool = None,
+        read_only: Optional[bool] = None,
     ) -> "View":
         """
         Returns a view that is limited to the specified bounding box.
@@ -268,11 +223,81 @@ class View:
             mag_view_bbox_at_creation=self._mag_view_bounding_box_at_creation,
         )
 
+    def get_buffered_slice_writer(
+        self,
+        offset: Vec3IntLike = Vec3Int(0, 0, 0),
+        buffer_size: int = 32,
+        dimension: int = 2,  # z
+    ) -> "BufferedSliceWriter":
+        """
+        The BufferedSliceWriter buffers multiple slices before they are written to disk.
+        The amount of slices that get buffered is specified by `buffer_size`.
+        As soon as the buffer is full, the data gets written to disk.
+
+        The user can specify along which dimension the data is sliced by using the parameter `dimension`.
+        To slice along the x-axis use `0`, for the y-axis use `1`, or for the z-axis use `2` (default: dimension=2).
+
+        The BufferedSliceWriter must be used as context manager using the `with` syntax (see example below),
+        which results in a generator consuming np.ndarray-slices via `writer.send(slice)`.
+        Exiting the context will automatically flush any remaining buffered data to disk.
+
+        Usage:
+        data_cube = ...
+        view = ...
+        with view.get_buffered_slice_writer() as writer:
+            for data_slice in data_cube:
+                writer.send(data_slice)
+
+        """
+        from webknossos.dataset._utils.buffered_slice_writer import BufferedSliceWriter
+
+        return BufferedSliceWriter(
+            view=self,
+            offset=Vec3Int(offset),
+            buffer_size=buffer_size,
+            dimension=dimension,
+        )
+
+    def get_buffered_slice_reader(
+        self,
+        offset: Vec3IntLike = Vec3Int(0, 0, 0),
+        size: Optional[Vec3IntLike] = None,
+        buffer_size: int = 32,
+        dimension: int = 2,  # z
+    ) -> "BufferedSliceReader":
+        """
+        The BufferedSliceReader yields slices of data along a specified axis.
+        Internally, it reads multiple slices from disk at once and buffers the data.
+        The amount of slices that get buffered is specified by `buffer_size`.
+
+        The user can specify along which dimension the data is sliced by using the parameter `dimension`.
+        To slice along the x-axis use `0`, for the y-axis use `1`, or for the z-axis use `2` (default: dimension=2).
+
+        The BufferedSliceReader must be used as a context manager using the `with` syntax (see example below).
+        Entering the context returns a generator with yields slices (np.ndarray).
+
+        Usage:
+        view = ...
+        with view.get_buffered_slice_reader() as reader:
+            for slice_data in reader:
+                ...
+
+        """
+        from webknossos.dataset._utils.buffered_slice_reader import BufferedSliceReader
+
+        return BufferedSliceReader(
+            view=self,
+            offset=Vec3Int(offset),
+            size=Vec3Int(size) if size is not None else self.size,
+            buffer_size=buffer_size,
+            dimension=dimension,
+        )
+
     def _assert_bounds(
         self,
         offset: Vec3Int,
         size: Vec3Int,
-        strict: bool = None,
+        strict: Optional[bool] = None,
     ) -> None:
         if strict is None:
             strict = self._is_bounded
@@ -332,9 +357,7 @@ class View:
         job_args = []
 
         for i, chunk in enumerate(
-            BoundingBox(view.global_offset, view.size).chunk(
-                chunk_size, list(chunk_size)
-            )
+            BoundingBox(view.global_offset, view.size).chunk(chunk_size, chunk_size)
         ):
             relative_offset = chunk.topleft - view.global_offset
             chunk_view = view.get_view(
@@ -411,10 +434,10 @@ class View:
 
         job_args = []
         source_chunks = BoundingBox(source_offset, source_view.size).chunk(
-            source_chunk_size, source_chunk_size.to_list()
+            source_chunk_size, source_chunk_size
         )
         target_chunks = BoundingBox(target_offset, target_view.size).chunk(
-            target_chunk_size, target_chunk_size.to_list()
+            target_chunk_size, target_chunk_size
         )
 
         for i, (source_chunk, target_chunk) in enumerate(
@@ -521,6 +544,9 @@ class View:
         return self.header.voxel_type
 
     def __enter__(self) -> "View":
+        warnings.warn(
+            "[DEPRECATION] Entering a View to open it is deprecated. The internal dataset will be opened automatically."
+        )
         return self
 
     def __exit__(
@@ -529,7 +555,7 @@ class View:
         _value: Optional[BaseException],
         _tb: Optional[TracebackType],
     ) -> None:
-        self.close()
+        pass
 
     def __repr__(self) -> str:
         return repr(
@@ -542,6 +568,32 @@ class View:
         assert self._mag_view_bbox_at_creation is not None
         return self._mag_view_bbox_at_creation
 
+    @property
+    def _wkw_dataset(self) -> wkw.Dataset:
+        if self._cached_wkw_dataset is None:
+            self._cached_wkw_dataset = Dataset.open(
+                str(self._path)
+            )  # No need to pass the header to the wkw.Dataset
+        return self._cached_wkw_dataset
+
+    @_wkw_dataset.deleter
+    def _wkw_dataset(self) -> None:
+        if self._cached_wkw_dataset is not None:
+            self._cached_wkw_dataset.close()
+            self._cached_wkw_dataset = None
+
+    def __del__(self) -> None:
+        del self._cached_wkw_dataset
+
+    def __getstate__(self) -> Dict[str, Any]:
+        d = dict(self.__dict__)
+        del d["_cached_wkw_dataset"]
+        return d
+
+    def __setstate__(self, d: Dict[str, Any]) -> None:
+        d["_cached_wkw_dataset"] = None
+        self.__dict__ = d
+
 
 def _assert_positive_dimensions(offset: Vec3Int, size: Vec3Int) -> None:
     if any(x < 0 for x in offset):
